using System;
using System.Windows.Forms;
using System.Collections;

namespace MDIApp.Syntax
{
    /// <summary>
    /// 
    /// </summary>
    public class Block : IComparable
    {
        private BlockList parent = null;

        private int begin;
        private int end;
        private int type;

        public int Index 
        {
            get 
            {
                return parent.IndexOf(this);
            }
        }

        public int Begin
        {
            get 
            {
                return this.begin;
            }
            set
            {
                begin = value;
            }
        }

        public int End
        {
            get 
            {
                return this.end;
            }
            set
            {
                end = value;
            }
        }

        public int Type
        {
            get 
            {
                return this.type;
            }
            set
            {
                type = value;
            }
        }


        private bool repaint;
        
        #region  
        public Block()
            : this(0, 0, 0, false) {}

        public Block(int begin, int end) 
            : this(begin, end, 0, true) {}

        public Block(int begin, int end, int type)
            : this(begin, end, type, true) {}
    
        public Block(ref BlockList parent, int begin, int end)
            : this(ref parent, begin, end, 0, true) {}

        public Block(ref BlockList parent, int begin, int end, int type)
            : this(ref parent, begin, end, type, true) {}

        public Block(ref BlockList parent)
            : this(ref parent, 0, 0, 0, false) {}

        public Block(ref BlockList parent, int begin, int end, int type, bool repaint)
            : this(begin, end, type, repaint) { this.parent = parent; }

        public Block(int begin, int end, int type, bool repaint)
        {
            this.begin = begin;
            this.end   = end;
            this.type  = type;
            this.repaint = repaint;
    }
    
        public Block(Block s) 
            : this(ref s.parent, s.begin, s.end, s.type, s.repaint) {
        parent = parent;
        }
        #endregion 

        public void SetParent(ref BlockList parent)
        {
            this.parent = parent;
        }
        
        public void mergeWithLeft() 
        {
            mergeWithLeft(false);
        }

        public void mergeWithRight()
        {
            mergeWithRight(false);
        }

        public void mergeWithLeft(bool repaint) 
        {
            if (Index != 0)
                begin = parent[Index -1].begin;
            this.repaint = repaint;
        }

        public void mergeWithRight(bool repaint)
        {
            if (Index != parent.Count-1)
                end = parent[Index+1].end;
            this.repaint = repaint;
        }

        public void MoveRight(int n)
        {
            this.begin += n;
            this.end += n;
        }


        public void Collapse(bool start)
        {
            if (start)
                begin = end;
            else
                end = begin;
        }


        public override string ToString()
        {
            if (parent != null)
                return 
                    "begin: " + "\"" + this.begin.ToString() + "\"\n" + 
                    "end: " + "\"" + this.type.ToString() + "\"\n" + 
                    "type: " + "\"" + this.type.ToString() + "\"\n" +
                    "index: " + "\"" + this.Index.ToString() + "\"\n";
            else
                return 
                    "begin: " + "\"" + this.begin.ToString() + "\"\n" + 
                    "end: " + "\"" + this.type.ToString() + "\"\n" + 
                    "type: " + "\"" + this.type.ToString() + "\"\n" +
                    "parent: is null";
        }
        
        #region IComparable Members
        public int CompareTo(Object obj)
        {
            if (obj is Block) 
            {
                int a = this.begin.CompareTo(((Block)obj).begin);
                int b = this.end.CompareTo(((Block)obj).end);

                if (a == b) 
                    return a;
                
                a = b;
            }
            obj.ToString();
            return 0;
        }

        #endregion
    }
}
