using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.Collections;

using MDIApp.Syntax;

namespace MDIApp.GUI
{
    public class Document : Form 
    {
        #region     


        private string oldText;
        private int selStart;
        private int selLength;
        private MDIApp.GUI.AdvTextBox editor;
        
        private string path;

        private BlockList blocks;
        private GroupList language;
		private Parser parser;		


        /// <summary>
        ///    Required designer variable.
        /// </summary>
        //        private System.ComponentModel.Container components;

        public Document(string docName) : base() 
        {
            //
            // Required for Windows Form Designer support
            //
            InitializeComponent();

            this.path = Application.ExecutablePath.Remove(
                Application.ExecutablePath.LastIndexOf("\\"),
                Application.ExecutablePath.Length - Application.ExecutablePath.LastIndexOf("\\")) + "\\";
            
            this.editor.TextChanged += new System.EventHandler(this.editor_TextChanged);

            this.editor.MouseUp += new MouseEventHandler(this.editor_MouseUp);
            this.editor.KeyUp += new KeyEventHandler(this.editor_KeyUp);

            oldText = "";

            language = new GroupList();
            language.SetZis(ref language);
            parser = new Parser(ref language);
			parser.log.BeginLog();
			blocks = new BlockList();
            blocks.Add(new Block(ref blocks));
		}

        #region Windows Form Designer generated code

        #region Dispose
        /// <summary>
        ///    Clean up any resources being used.
        /// </summary>
        protected override void Dispose(bool disposing)
        {
            if (disposing) 
            {
                /*                if (components != null) 
                                {
                                    components.Dispose();
                                }
                */            }
			parser.log.EndLog();
            base.Dispose(disposing);
        }
        #endregion

        /// <summary>
        ///    Required method for Designer support - do not modify
        ///    the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
			this.editor = new MDIApp.GUI.AdvTextBox();
			this.SuspendLayout();
			// 
			// editor
			// 
			this.editor.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right);
			this.editor.Font = new System.Drawing.Font("Courier New", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(204)));
			this.editor.ForeColor = System.Drawing.SystemColors.WindowText;
			this.editor.Name = "editor";
			this.editor.Size = new System.Drawing.Size(560, 314);
			this.editor.TabIndex = 2;
			this.editor.Text = "";
			// 
			// Document
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(560, 301);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.editor});
			this.Name = "Document";
			this.ResumeLayout(false);

		}

        #endregion

        private void buttonFormat_Click(object sender, System.EventArgs e)
        {
            language.Clear();
            language.LoadFromFile("wordfile.txt");
            
            string values = "";
            
            foreach (MDIApp.Syntax.Group group in language)
            {
                values += group.ToString();
            }

            MessageBox.Show(values);
        }

        private void editor_CursorMoved() 
        {
            selStart = editor.SelectionStart;
            selLength = editor.SelectionLength;
        }

        private void editor_KeyUp(object sender, KeyEventArgs e)
        {
            editor_CursorMoved();
        }
        
        private void editor_MouseUp(object sender, MouseEventArgs e)
        { 
            editor_CursorMoved(); 
        }

        #endregion     

        private void editor_TextChanged(object sender, System.EventArgs e)
        {
			editor._Paint = false;

			int ss = editor.SelectionStart;
			int sl = editor.SelectionLength;
			
			if (editor.Text.Length < oldText.Length) 
			{
				Block head = null, tail = null;
				foreach(Block block in blocks)
				{
					// ...
					if ((head == null) && (block.Begin <= 0) && (block.End >= 0)) 
						head = block;
					// ... 
					if ((tail == null) && (block.Begin <= (editor.Text.Length)) && (block.End >= (editor.Text.Length))) 
						tail = block;
                
					//      
					if ((tail != null) && (head != null)) 
						break;
				}
            
				int headIndex = head.Index;
				int offset = head.Begin;

				#region  ,   head  tail
				if ((tail.Index-head.Index-2) > 0 ) 
					blocks.RemoveRange(head.Index+1, tail.Index-head.Index-1);
				#endregion

				/*	foreach (Block block in blocks) 
				{
					blocks.Remove(block);
				}
		*/
				//  
				BlockList list = parser.Parse(editor.Text);
            		
				if (head.Equals(tail))
					blocks.RemoveAt(head.Index);
				else
				{
					blocks.RemoveAt(tail.Index);
					blocks.RemoveAt(head.Index);
				}

				foreach(Block block in list) 
				{
					block.SetParent(ref blocks);
					block.MoveRight(offset);
				}
				blocks.InsertRange(headIndex, list);

				foreach (Block block in blocks.GetRange(headIndex, list.Count))
				{
					editor.SelectionStart = block.Begin;
					editor.SelectionLength = block.End;
					editor.SelectionColor = parser.language[block.Type].Color;
				}
		
				//     
				editor.SelectionStart = ss;
				editor.SelectionLength = sl;
			
				editor_CursorMoved();

				oldText = editor.Text;
			
				editor._Paint = true;
			
			} 
			else 
			{
				Block head = null, tail = null;

            #region     
				foreach(Block block in blocks)
				{
					// ...
					if ((head == null) && (block.Begin <= selStart) && (block.End >= selStart)) 
						head = block;
					// ... 
					if ((tail == null) && (block.Begin <= (selStart+selLength)) && (block.End >= (selStart+selLength))) 
						tail = block;
                
					//      
					if ((tail != null) && (head != null)) 
						break;
				}
            #endregion
            
			#region       
				if (head.Equals(tail))
				{
					//      head
					if (head.End == selStart + selLength) 
					{
						if (head.Index < (blocks.Count-1))
						{
							tail = blocks[head.Index+1];
						}
					}
		        }
				else 
                #region  ,   head  tail
					if ((tail.Index-head.Index-2) > 0 )
					blocks.RemoveRange(head.Index+1, tail.Index-head.Index-1);
                #endregion
            #endregion 

				int delta = editor.Text.Length - oldText.Length;
				int offset = head.Begin;
            
            #region    
				if (!head.Equals(tail))
					tail.MoveRight(delta);
				else
					head.End += delta;

				for (int i = tail.Index+1; i < blocks.Count; i++) 
					blocks[i].MoveRight(delta);
            #endregion
			
				//  
				BlockList list = parser.Parse(editor.Text.Substring(offset, tail.End - offset));
            		
				int headIndex = head.Index;

				if (head.Equals(tail))
					blocks.RemoveAt(head.Index);
				else
				{
					blocks.RemoveAt(tail.Index);
					blocks.RemoveAt(head.Index);
				}

				foreach(Block block in list) 
				{
					block.SetParent(ref blocks);
					block.MoveRight(offset);
				}
				blocks.InsertRange(headIndex, list);

				foreach (Block block in blocks.GetRange(headIndex, list.Count))
				{
					editor.SelectionStart = block.Begin;
					editor.SelectionLength = block.End;
					editor.SelectionColor = parser.language[block.Type].Color;
             	}
       
				//     
				editor.SelectionStart = ss;
				editor.SelectionLength = sl;
			
				editor_CursorMoved();

				oldText = editor.Text;
			
				editor._Paint = true;
			
			}
        }
    }
}
